﻿using ProjectAirBomber.Drawnings;
using ProjectAirBomber.MovementStrategy;

namespace ProjectAirBomber;
/// <summary>
/// Форма работы с объектом "Бомбардировщик"
/// </summary>
public partial class FormAirBomber : Form
{
    /// <summary>
    /// Поле-объект для прорисовки объекта
    /// </summary>
    private DrawningBomber? _drawningBomber;

    /// <summary>
    /// Стратегия перемещения
    /// </summary>
    private AbstractStrategy? _strategy;


    /// <summary>
    /// Получение объекта 
    /// </summary>
    public DrawningBomber SetBomber
    {
        set
        {
            _drawningBomber = value;
            _drawningBomber.SetPictureSize(pictureBoxAirBomber.Width, pictureBoxAirBomber.Height);
            comboBoxStrategy.Enabled = true;
            _strategy = null;
            Draw();
        }
    }

    /// <summary>
    /// Конструктор формы
    /// </summary>
    public FormAirBomber()
    {
        InitializeComponent();
        _strategy = null;
    }

    /// <summary>
    /// Метод прорисовки бомбардировщика
    /// </summary>
    private void Draw()
    {
        if (_drawningBomber == null)
        {
            return;
        }

        Bitmap bmp = new(pictureBoxAirBomber.Width, pictureBoxAirBomber.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningBomber.DrawTransport(gr);
        pictureBoxAirBomber.Image = bmp;
    }

    /// <summary>
    /// Перемещение объекта по форме (нажатие кнопок навигации)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningBomber == null)
        {
            return;
        }

        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningBomber.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningBomber.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawningBomber.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawningBomber.MoveTransport(DirectionType.Right);
                break;
        }

        if (result)
        {
            Draw();
        }
    }
    /// <summary>
    /// Обработка нажатия кнопки "Шаг"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningBomber == null)
        {
            return;
        }

        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };
            if (_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveableBomber(_drawningBomber), pictureBoxAirBomber.Width, pictureBoxAirBomber.Height);
        }

        if (_strategy == null)
        {
            return;
        }

        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if (_strategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }
    }
}