﻿using ProjectAirBomber.Drawnings;


namespace ProjectAirBomber.MovementStrategy;
public class MoveableBomber : IMoveableObject
{
    /// <summary>
    /// Поле-объект класса DrawningTrans или его наследника
    /// </summary>
    private readonly DrawningBomber? _bomber = null;
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="bomber">Объект класса DrawningTrans</param>
    public MoveableBomber(DrawningBomber bomber)
    {
        _bomber = bomber;
    }

    public ObjectParameters? GetObjectPosition
    {
        get
        {
            if (_bomber == null || _bomber.EntityBomber == null ||
            !_bomber.GetPosX.HasValue || !_bomber.GetPosY.HasValue)
            {
                return null;
            }
            return new ObjectParameters(_bomber.GetPosX.Value,
            _bomber.GetPosY.Value, _bomber.GetWidth, _bomber.GetHeight);
        }
    }
    public int GetStep => (int)(_bomber?.EntityBomber?.Step ?? 0);
    public bool TryMoveObject(MovementDirection direction)
    {
        if (_bomber == null || _bomber.EntityBomber == null)
        {
            return false;
        }
        return _bomber.MoveTransport(GetDirectionType(direction));
    }
    /// <summary>
    /// Конвертация из MovementDirection в DirectionType
    /// </summary>
    /// <param name="direction">MovementDirection</param>
    /// <returns>DirectionType</returns>
    private static DirectionType GetDirectionType(MovementDirection direction)
    {
        return direction switch
        {
            MovementDirection.Left => DirectionType.Left,
            MovementDirection.Right => DirectionType.Right,
            MovementDirection.Up => DirectionType.Up,
            MovementDirection.Down => DirectionType.Down,
            _ => DirectionType.Unknow,
        };
    }


}
